
% Robust control and optimal monetary policy inertia
% in a small-open new Keynesian model

% Daisuke Ida and Mitsuhiro Okano

% December 9, 2021

% housekeeping
clear
close

setpath_io

% calc computation time
tic

% # of variables
n1 = 3; % # of predetermined variables
n2 = 2; % # of jump variables
n3 = 1; % # of instrument variables
n4 = 2; % # of shock variables

% Target variables: Y(t) = Cx x(t) + Cu u(t)
ny = 2; % # of target variables in the loss function
nuu = 1; % # of instrument variables in the loss function

n = n1+n2;

% parameters
params; % call param.m

theta_vec = [1000 750 500 400 300 200 100 80];
num_tht = size(theta_vec,2);

% variance-covariance matrix
SigmaXX = zeros(n1,n1); SigmaXX(1,1) = sdev_a^2; SigmaXX(2,2) = sdev_u^2;

% weight on true loss function
lambda_x = 0.25;

% calculate structural parameters
[var_omega,frac_phi,Phi,sigma_nu,zetta,kappa_nu] = calc_deep_param(sigma,varphi,eta,theta,alpha,nu,betta);

% calculate structural matrices
[A0,A,B,Z,Cx,Cu,Q,U,R] = calc_matrix(n,n3,ny,nuu,betta,rho_a,rho_u,sigma_nu,kappa_nu,lambda_x);

% Robust approach
C  =  [ zeros(n1); ...
            zeros(n2,n1) ];
C(1,1) = 1; C(2,2) = 1;

% shock vector
uy  = A0\[sdev_a;0;0;0;0];  % positive demand shock
upi = A0\[0;sdev_u;0;0;0];  % mark-up shock

% option for commitment
cutoff = 1.0;

% store results
var_com_t = zeros(num_tht,2); var_disc_t = zeros(num_tht,2);
lossd_nit_t = zeros(num_tht,2); lossd_slp_t = zeros(num_tht,2);
lossd_it_t = zeros(num_tht,2); lossd_rex_t = zeros(num_tht,2);

for k = 1:num_tht
    
    theta_evil = theta_vec(k);
    
    call_policy_implement; % call_policy_implement.m
    
    var_com_t(k,:) = var_com;
    var_disc_t(k,:) = var_disc;
    lossd_nit_t(k,:) = lossd_nit; 
    lossd_slp_t(k,:) = lossd_slp;
    lossd_it_t(k,:) = lossd_it;
    lossd_rex_t(k,:) = lossd_rex;
    
end

disp('****************************************************')
disp('Showing results')

fprintf('\n')
disp('*** Relative welfare losses under worst case (commitment: 100) ***')
disp('     PD       IT        SLP       NIGT      RERT')

for kk = 1:num_tht
disp([100*var_disc_t(kk,1)/var_com_t(kk,1) 100*lossd_it_t(kk,1)/var_com_t(kk,1) 100*lossd_slp_t(kk,1)/var_com_t(kk,1)...
    100*lossd_nit_t(kk,1)/var_com_t(kk,1) 100*lossd_rex_t(kk,1)/var_com_t(kk,1)]);
end

fprintf('\n')
disp('*** Welfare losses under approximating case (commitment 100) ***')
disp('     PD       IT        SLP       NIGT      RERT')

for kk = 1:num_tht
disp([100*var_disc_t(kk,2)/var_com_t(kk,2) 100*lossd_it_t(kk,2)/var_com_t(kk,2) 100*lossd_slp_t(kk,2)/var_com_t(kk,2)...
    100*lossd_nit_t(kk,2)/var_com_t(kk,2) 100*lossd_rex_t(kk,2)/var_com_t(kk,2)]);
end

fprintf('\n')
disp('******************************************************')


toc