function [M,N,Ma,Na,Fv,J0,J0a] = ComAlgR( A,B,Q,R,U,bet,n1,n2,cutoff,C,theta );
%ComAlgR     Solves for robust optimization problem in commitment case.
%
%            The economy evolves according to
%
%            [x1(t+1);E(t)x2(t+1)] = A*[x1(t);x2(t)] + B*u(t) + C*e(t+1),
%
%            where x1(t) ("backward looking") has n1 elements,
%            and x2(t) ("forward looking") has n2 elements. The initial
%            value of x1, x1(0), is given by history. Let n=n1+n2.
%            u(t) is an kx1 vector of decision variables, which follows
%            the decision rule.
%
%            Define x(t)=[x1(t),x2(t)]. The loss function of the policy
%            maker is
%
%            Sum{ (bet^t)*[x(t)'Q*x(t)+2x(t)'U*u(t)+u(t)'R*u(t)],t=0,1,... }
%
%
%
%  Usage:    [M,N,Ma,Na,Fv,J0,J0a] = ComAlgR( A,B,Q,R,U,bet,n1,n2,cutoff,C,theta );
%
%
%  Input:    A        nxn matrix, (n=n1+n2)
%            B        nxk matrix
%            Q        nxn matrix, symmetric
%            R        kxk matrix, symmetric
%            U        nxk matrix
%            bet      scalar, discount factor (eg 0.99)
%            n1       scalar, # of predetermined variables
%            n2       scalar, # of forward looking variables
%            cutoff   scalar, max modulus of stable roots, eg. 1.0001
%            C        nxn1 matrix. CC' is the VCV matrix of the errors;
%                     the last n2 rows are zeros.
%            theta    scalar, (inverse of the) degree of robustness
%
%  Output:   M        nxn matrix, [x1(t+1),p2(t+1)] = M*[x1(t),p2(t)] + C*e(t+1)
%                     in the worst case solution, where p are multipliers.
%            N        (n2+k+n1*2)x(n1*2) matrix, [x2;u;v;p1]= N*[x1;p2], worst case
%            Ma       nxn matrix, [x1(t+1),p2(t+1)] = Ma*[x1(t),p2(t)] + C*e(t+1)
%                     in the approximating solution
%            Na       (n2+k+n1*2)x(n1*2) matrix, [x2;u;v;p1]= N*[x1;p2], approximating case
%            Fv       n1x(n1*2) matrix, v = -Fv*(x1;p2)
%            J0       scalar, loss function value, worst case
%            J0a      scalar, loss function value, approximating model
%
%
%  Note:     N and Na differs only in that rows corrsponding to v are zero in Na.
%
%  Calls on: ComItAlg
%
%
%  Paolo Giordani and Paul.Soderlind@unisg.ch, Jun 2001
%------------------------------------------------------------------------------

Q = (Q + Q')/2;                 %to make symmetric
R = (R + R')/2;

k = size(R,1);                  %no of control variables
n = n1 + n2;

if any(any(C(n1+1:n,:)));
  warning('Non-zero elements found in the last n2 rows of C');
  return;
end;

Btilde = [ B, C ];
Utilde = [ U, zeros(n1+n2,n1) ];
Rtilde = [ R,            zeros(k,n1); ...
           zeros(n1,k),  -theta*eye(n1) ];


[M,N] = ComItAlg( A,Btilde,Q,Rtilde,Utilde,bet,n1,n2,cutoff );   %commitment, robust policy


N1 = N(1:n2,:);                         %x2 = N1*[x1;p2]
N2 = N(n2+1:n2+k,:);                    % u = N2*[x1;p2]

N3 = N(n2+k+1:n2+k+n1,:);               % v = N3*[x1;p2]

P = [ eye(n1), zeros(n1,n2); ...
      N1                   ];            %[x1;x2] = P*[x1;p2]
Fu  = -N2*inv(P);                        %u = -Fu*[x1;x2]
Fv  = -N3*inv(P);                        %v = -Fv*[x1;x2], zeros if RobustIt not 1

Mw = inv(P)*(A-B*Fu-C*Fv)*P;     %alternative expression for M, should equal M
%disp(' ');disp('M-Mw');disp([M-Mw]);

Ma = inv(P)*(A-B*Fu)*P;          %[x1(t+1);p2(t+1)] = Ma*[x1(t);p2(t)], approximate model
Na = N;                          %[x2;u;v;p1] = Na*[x1;p2], approximate model
Na(n2+k+1:n2+k+n1,:) = zeros(n1,n1+n2);  %no evil shocks in approximate model
%disp(' ');disp('M-Ma');disp([M-Ma]);

maxEig = max(abs(eig(Ma)));
if maxEig > cutoff;
  warning('Some abs(eigenvalue) of Ma > cutoff');
end;


Ptilde = [ eye(n1+n2); -Fu; -Fv ] * P;             %value function, worst case
W      = Ptilde' * [ Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n*n) - kron(M',bet*M'))\W(:);
V      = reshape(vecV,n,n);                 %solves V = W + bet*M'*V*M
if bet == 1;
  J0 = trace(C'*V*C*eye(n1));
else;
  J0 = bet/(1-bet) * trace(C'*V*C*eye(n1));
end;


Ptilde = [ eye(n1+n2); -Fu; zeros(n1,n1+n2) ] * P;   %value function, approximating model
W      = Ptilde' * [ Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n*n) - kron(Ma',bet*Ma'))\W(:);
V      = reshape(vecV,n,n);                 %solves V = W + bet*Ma'*V*Ma
if bet == 1;
  J0a = trace(C'*V*C*eye(n1));
else;
  J0a = bet/(1-bet) * trace(C'*V*C*eye(n1));
end;
%-----------------------------------------------------------------------
