function [M,N,Ma,Na,Fu,Fv,J0,J0a] = DiscAlgR( A,B,Q,R,U,bet,n1,n2,Vt1,Ct1,...
                                              ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter,...
                                              C,theta );
%DiscAlgR    Solves for robust optimization problem in discretionary case.
%
%            Same economy and loss function as in ComAlgR, but discretionary
%            equilibrium.
%
%
%
%  Usage:    [M,N,Ma,Na,Fu,Fv,J0,J0a] = DiscAlgR( A,B,Q,R,U,bet,n1,n2,Vt1,Ct1, ...
%                                                 ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter,...
%                                                 C,theta );
%
%
%  Input:    A        see ComAlgR
%            B
%            Q
%            R
%            U
%            bet
%            n1
%            n2
%            cutoff
%            Vt1      n1xn1 matrix. Initial guess for the value function
%                     (a positive definite matrix, such as the identity matrix)
%            Ct1      Initial guess for the matrix N. Ex: an n2xn1 zero matrix
%            ConvCrit scalar, convergence criterion (a small number, such as 1e-6)
%            Vweight  scalar, weight of value function in convergence test
%            Fweight  scalar, weight of policy function in convergence test
%            CritLags scalar, typically 1. If set to x, the convergence criterion is
%                     based on comparison of iteration t with iterations t-1,...,t-x
%            step     scalar in (0,1): factor of updating of V and F as
%                     in Vt1 = step*V + (1-step)*Vt1
%            PrintIt  scalar, 1: printing iteration number 2: printing iteration number
%                     and convergence criteria
%            MaxIter  scalar, maximum number of iterations (eg. 10000). NEW (March 2003)
%            C        see ComAlgR
%            theta

%
%  Output:   M        n1xn1 matrix, x1(t+1) = M*x1(t) + C1*e(t+1) in the worst
%                     case solution, C1 is the first n1 rows of C
%            N        n2xn1 matrix, x2 = N*x1
%            Ma       n1xn1 matrix, x1(t+1) = M*x1(t) + C1*e(t+1) in the approximating
%                     solution
%            Na       n2xn1 matrix, x2 = Na*x1, approximating model
%            Fu       kxn1 matrix, u = -Fu*x1
%            Fv       n1xn1 matrix, v = -Fv*x1
%            J0       scalar, loss function value, worst case
%            J0a      scalar, loss function, approximating model
%
%
%  Note:     The Na matrix (x2 = Na*x1) in the approximating model is the same as in the
%            worst case.
%
%  Calls on: DiscAlg
%
%
%  Paolo Giordani and Paul.Soderlind@unisg.ch, Sep 2001
%------------------------------------------------------------------------------

Q = (Q + Q')/2;                %to make symmetric
R = (R + R')/2;

k = size(R,1);
n = n1 + n2;

if any(any(C(n1+1:n,:)));
  warning('Non-zero elements found in the last n2 rows of C');
  return;
end;


Btilde = [ B, C ];
Utilde = [ U, zeros(n1+n2,n1) ];
Rtilde = [ R,            zeros(k,n1); ...
           zeros(n1,k),  -theta*eye(n1) ];


[M,N,V,F] = DiscAlg( A,Btilde,Q,Rtilde,Utilde,bet,n1,n2,Vt1,Ct1, ...
                     ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter );

A11 = A(1:n1,1:n1);
A12 = A(1:n1,(n1+1):n);
A21 = A((n1+1):n,1:n1);
A22 = A((n1+1):n,(n1+1):n);

B1 = B(1:n1,:);
B2 = B(n1+1:n,:);

C1 = C(1:n1,:);
C2 = C(n1+1:n,:);

Fu = F(1:k,:);
Fv = F(k+1:k+n1,:);

Mw  = A11 + A12*N - B1*Fu - C1*Fv;
NMw = A21 + A22*N - B2*Fu;


Ma  = A11 + A12*N - B1*Fu;
Na  = N;


Ptilde = [ eye(n1); N; -Fu; -Fv ];             %value function, worst case
W      = Ptilde' * [Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n1*n1) - kron(M',bet*M'))\W(:);
V      = reshape(vecV,n1,n1);                 %solves V = W + bet*M'*V*M
if bet == 1;
  J0 = trace(C1'*V*C1*eye(n1));
else;
  J0 = bet/(1-bet) * trace(C1'*V*C1*eye(n1));
end;


Ptilde = [ eye(n1); Na; -Fu; zeros(n1,n1) ];             %value function
W      = Ptilde' * [Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n1*n1) - kron(Ma',bet*Ma'))\W(:);
V      = reshape(vecV,n1,n1);                 %solves V = W + bet*Ma'*V*Ma
if bet == 1;
  J0a = trace(C1'*V*C1*eye(n1));
else;
  J0a = bet/(1-bet) * trace(C1'*V*C1*eye(n1));
end;
%-----------------------------------------------------------------------


%  OLD STUFF

% Vs_1  = 0;
% Vb     = 0;
% Vdiff = 100;
% while Vdiff > 1E-6;    %iterating on value matrix V
%   Vb     = W + bet*M'*Vs_1*M;  %J(0) = Sum[(bet^t)z(t)'Wz(t),t=0,...] = z(0)'*V*z(0) + const
%   Vdiff = max(  max( abs(Vs_1-Vb) )  );
%   Vs_1  = Vb;            %Vs_1 is V(s-1), V is V(s)
% end;
% disp('testing V ');disp([V,V-Vb]);

% Vs_1   = 0;
% Vb     = 0;
% Vdiff  = 100;
% while Vdiff > 1E-6;
%   Vb    = W + bet*Ma'*Vs_1*Ma;
%   Vdiff = max(  max( abs(Vs_1-Vb) )  );
%   Vs_1  = Vb;
% end;
% disp('testing V ');disp([V,V-Vb]);
