function [M,N,Ma,Na,J0,J0a] = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,x10,cutoff,C,theta,Fv,J0aIt );
%SimpAlgR    Solving robust RE model for given simple rules of both policy maker
%            and evil agent.
%
%            Same economy and loss function as in ComAlgR, but
%            policy is now postulated to follow
%
%            u(t) = -Fu*x(t), where Fu is a given kxn matrix.
%
%
%  Usage:    [M,N,Ma,Na,J0,J0a] = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,x10,cutoff,C,theta,Fv,J0aIt );
%
%
%  Input:    A        see ComAlgR
%            B
%            Q
%            R
%            U
%            bet
%            n1
%            n2
%            Fu       kxn matrix, policy maker's decision rule u = -Fu*x
%            x10      n1x1, initial values of x1(t) vector
%            cutoff   same as ComAlgR
%            C        see ComAlgR
%            theta
%            Fv       n1xn matrix, evil agent's decision rule v = -Fv*x, last n2 columns
%                     should be zero
%            J0aIt    scalar, if 1: calculate loss for approx model
%
%  Output:   M        n1xn1 matrix, x1(t+1) = M*x1(t) + C1*e(t+1), worst case solution
%            N        n2xn1 matrix, x2 = N*x1, worst case
%            Ma       n1xn1 matrix, x1(t+1) = Ma*x1(t) + C1*e(t+1), approximating model
%            Na        n2xn1 matrix, x2 = Na*x1,  approximating model
%            J0       scalar, loss function value, worst case
%            J0a      scalar, loss function value, approximating case
%
%
%  Note:     The Na matrix (x2 = Na*x1) in the approximating model is the same as in the
%            worst case.
%
%
%
%  Calls on: SimpRulT
%
%
%  Paolo Giordani and Paul.Soderlind@unisg.ch, Sep 2001
%------------------------------------------------------------------------------

Q = (Q + Q')/2;                %to make symmetric
R = (R + R')/2;

k = size(R,1);
n = n1 + n2;

if any(any(Fv(:,n1+1:n)));
  warning('Non-zero elements found in the last n2 columns of Fv');
  return;
end;

if any(any(C(n1+1:n,:)));
  warning('Non-zero elements found in the last n2 rows of C');
  return;
end;


Btilde = [ B, C ];
Utilde = [ U, zeros(n1+n2,n1) ];
Rtilde = [ R,            zeros(k,n1); ...
           zeros(n1,k),  -theta*eye(n1) ];

C1 = C(1:n1,:);


[M,N,J0] = SimpRulT(A,Btilde,Q,Rtilde,Utilde,bet,n1,n2,[Fu;Fv],C1*C1',x10,cutoff);
if isnan(M);
  J0 = 0;          %evil agent wants to maximize the loss: 0 is bad for him
  return;
end;
%disp(M);

P   = [ eye(n1); N ];
MM  = (A - B*Fu - C*Fv)*P;
Mw  = MM(1:n1,:);
NMw = MM(n1+1:n,:);

% disp(' ');disp('M-Mw');disp([M-Mw]);
% disp(' ');disp('N*M-NMw');disp([N*M-NMw]);


MM  = (A - B*Fu)*P;
Ma  = MM(1:n1,:);
Na  = N;


if J0aIt == 1;            %calculate loss for approx model
  Ptilde = [ eye(n); -Fu; zeros(n1,n) ] * P;             %value function
  W      = Ptilde' * [ Q, Utilde; Utilde', Rtilde] * Ptilde;
  vecV   = (eye(n1*n1) - kron(Ma',bet*Ma'))\W(:);
  V      = reshape(vecV,n1,n1);                 %solves V = W + bet*Ma'*V*Ma
  if bet == 1;
      J0a = trace(C1'*V*C1*eye(n1));
  else;
      J0a = bet/(1-bet) * trace(C1'*V*C1*eye(n1));
  end;
else;
  J0a = NaN;
end;      %if J0aIt

%-----------------------------------------------------------------------
