function [M,C,J0] = SimpRulT( A,B,Q,R,U,bet,n1,n2,F,SigmaXX,x10,cutoff );
%SimpRulT   Solving RE model for given simple rule u(t)  = -F * x(t),
%           where   x(t)  = [x1(t)' x2(t)']'.
%
%
%
%  Usage:    [M,C,J0] = SimpRulT( A,B,Q,R,U,bet,n1,n2,F,SigmaXX,x10,cutoff );
%
%  Input:     A,B,Q,R,U,bet,n1,n2: see ComItAlg
%             F          kxn vector of decision rule
%             SigmaXX    n1xn1 covariance matrix of shocks
%             X_0        n1x1 vector, initial values for predetermined
%                        variables
%             cutoff     scalar, max modulus of backward looking variables
%
%  Output:    M          n1xn1 matrix, x1(t+1) = M*x1(t) + e(t+1)
%             C          n2xn1 matrix, x2(t) = C*x1(t)
%             J0         scalar, value of loss function
%
%
%
%  Calls on:  reorder (if MatLab)
%
%
%  Paul Soderlind, Paul.Soderlind@unisg.ch, Aug 2000, Feb 2003
%-----------------------------------------------------------------------

Q = (Q + Q')/2;                %to make symmetric
R = (R + R')/2;

A = A - B*F;                     %subst for u(t) in evolution
n = n1 + n2;


if exist('OCTAVE_VERSION');             %Octave, real generalized Schur, sort with abs(lambda)>=1 last
  [S,T,Z,lambda] = qz(eye(size(A)),A,'B');
  if length(lambda) < size(S,2);         %if missing eigenvalues, add them as 999 ('infinite')
    lambda = [lambda;999*ones(size(S,2)-length(lambda),1)];
  end;
  logcon = 1 <= (abs(lambda)*cutoff);   %1 for stable eigenvalue
else;                                   %MatLab, complex generalized Schur decomposition
  [S,T,Qa,Z] = qz(eye(size(A)),A); %MatLab: I=Q'SZ' and A=Q'TZ'; Paul S:  I=QSZ' and A=QTZ',%but Q isn't used
  [S,T,Qa,Z] = reorder(S,T,Qa,Z);   % reordering of generalized eigenvalues, T(i,i)/S(i,i), in ascending order
  logcon = abs(diag(T)) <= (abs(diag(S))*cutoff);  %1 for stable eigenvalue
end;

if sum(logcon) < n1;
  warning('Too few stable roots: no stable solution');
  M = NaN; C = NaN;J0 = NaN;
  return;
elseif sum(logcon) > n1;
  warning('Too many stable roots: inifite number of stable solutions');
  M = NaN; C = NaN;J0 = NaN;
  return;
end;


Stt = S(1:n1,1:n1);
Zkt = Z(1:n1,1:n1);
Zlt = Z(n1+1:n,1:n1);
Ttt = T(1:n1,1:n1);


if cond(Zkt) > 1e+14;
  warning('Zkt is singular: rank condition for solution not satisfied');
  M = NaN; C = NaN;J0 = NaN;
  return;
end;

Zkt_1 = inv(Zkt);         %inverting
Stt_1 = inv(Stt);


M = real(Zkt*Stt_1*Ttt*Zkt_1);       %x1(t+1) = M*x1(t) + e(t+1)
C = real(Zlt*Zkt_1);                 %x2(t) = C*x1(t)

P = [ eye(n1);
      C;
      (-F*[eye(n1);C]) ];
QUUR = [ Q,  U;
         U', R ];
PQUURP = P'*QUUR*P;

vecV = (eye(n1*n1) - kron(M',bet*M'))\PQUURP(:);
V    = reshape(vecV,n1,n1);                 %solves V = W + bet*M'*V*M

J0 =  x10'*V*x10 + (bet/(1-bet)) * sum(diag(V*SigmaXX,0));
%----------------------------------------------------------------------


% OLD STUFF

% Vs1 = zeros(n1,n1);        %iterating on value matrix
% Vb = Vs1;
% Vdiff = 100;
% while Vdiff > 1E-6;
%   Vb = PQUURP + bet*M'*Vs1*M;
%   Vdiff = max(  max( abs(Vs1-Vb) )  );
%   Vs1 = Vb;
% end;
% disp('testing V ');disp([V,V-Vb]);

