function [theta, rhoval, JcMval] = findTheta( A,B,Q,R,U,bet,Fu,n1,n2,cutoff,C, ...
                                 T,NSim,sigma,randnstate,ModelType,Fv0,MaxIter,constant );
%findTheta    finds theta with detection error probablilities close to 0.2.
%             modified version of ErrDetProb.m by Giordani and Soderlind Sep 2001.
% 
%             Simulate error detection probabilites for various sigma = -1/theta.
%             Model/equilibrium is chosen by a switch
%
%
%
%
%  Usage:    [rho,JcM] = ErrDetProb( A,B,Q,R,U,bet,Fu,n1,n2,cutoff,C, ...
%                                    T,NSim,sigma,randnstate,ModelType,Fv0,constant );
%
%
%  Input:    A,B,Q,R,U,bet,Fu,n1,n2,cutoff,C: see ComAlgR, ComAlgSR, DiscAlgR, or SimpAlgR
%            T            scalar, simulated sample length, Ex. 142
%            NSim         scalar, # of simulations, eg 250
%            sigma        Kx1 vector, sigma = -1/theta values to calculate probs for,
%                         Ex. sigma = [ -0.04; -0.03; -0.02; -0.01 ];
%            randnstate   scalar, seed of random number generator, Ex. 123
%            ModelType    scalar, switch of model to simulate:
%                            1: ComAlgSR
%                            2: ComAlgR
%                            3: DiscAlgR
%                            4: SimpAlgR  (note: requires optimization routine, see SimpAlgROpt)
%            Fv0         matrix, used only for simple rule, see SimpAlgROpt
%            MaxIter     scalar, maximum number of iterations, used only for discretion, see DiscAlgR
%            constant    scalar, 1 if the first element of x is 1
%
%
%  Output:  rho          Kx1 vector, error detection probabilities
%           JcM          Kx2 matrix, constant in loss function, for
%                        [worst case, approximating model]
%
%
%
%  Calls on: ComAlgSR, ComAlgR, DiscAlgR, SimpAlgROpt, Var1SimPs
%
%
%  Orinigally
%  Paolo Giordani and Paul S�derlind, Paul.Soderlind@unisg.ch, Sep 2001
%
%  Modified by
%  Mitsuhiro Okano, okano@ogu.ac.jp, May 2021
%------------------------------------------------------------------------------

Kbig = length(sigma);

rho = zeros(Kbig,1);            %to put results in
rho(:) = NaN;
JcM = zeros(Kbig,2);
JcM(:) = NaN;

for k = 1:Kbig;                      %loop over theta = -1/sigma

  if ModelType == 1;                 %commitment, stable
    [Mw,Nw,Ma,Na,Fv,J0,J0a] = ComAlgSR( A,B,Q,R,U,bet,cutoff,C,-1/sigma(k) );
  elseif ModelType == 2;             %commitment, unstable
    [Mw,Nw,Ma,Na,Fv,J0,J0a] = ComAlgR(  A,B,Q,R,U,bet,n1,n2,cutoff,C,-1/sigma(k) );
  elseif ModelType == 3;             %discretion
    [Mw,Nw,Ma,Na,Fu,Fv,J0,J0a] = DiscAlgR( A,B,Q,R,U,bet,n1,n2,eye(n1),zeros(n2,n1),...
                                         1e-6,0,1,0,1,0,MaxIter,C,-1/sigma(k) );
    Fv = [ Fv, zeros(n1,n2) ];           %to make Fv n1xn as required below
  elseif ModelType == 4;
    [Mw,Nw,Ma,Na,J0,J0a,Fv] = SimpAlgROpt( Fv0,A,B,Q,R,U,bet,n1,n2,Fu,cutoff,C,-1/sigma(k) );
  end;

  if constant == 1;
    Mw2    = Mw(2:end,2:end);
    xmeanw = inv( eye(size(Mw2,1))-Mw2 ) * Mw(2:end,1);
    xmeanw = [1;xmeanw];
    Ma2    = Ma(2:end,2:end);
    xmeana = inv( eye(size(Ma2,1))-Ma2 ) * Ma(2:end,1);
    xmeana = [1;xmeana];
  else;
    xmeanw = zeros(size(Mw,1),1);
    xmeana = zeros(size(Ma,1),1);
  end;


  if exist('OCTAVE_VERSION');
    randn('seed',randnstate);
  else;
    randn('state',randnstate);
  end;


  rarwN = zeros(NSim,2);
  rarwN(:) = NaN;
  for i = 1:NSim;                                %simulate NSim times
    epsilon = randn(T+1,n1);                     %iid N(0,I)
    if ModelType == 1;
      Xw = Var1SimPs(Mw,epsilon*C',T)         + repmat(xmeanw',T,1);    %simulate VAR(1)
      Xa = Var1SimPs(Ma,epsilon*C',T)         + repmat(xmeana',T,1);
    elseif ModelType == 2;
      x1p2  = Var1SimPs(Mw,epsilon*C',T)      + repmat(xmeanw',T,1);
      x2up1 = x1p2*Nw';
      Xw    = [ x1p2(:,1:n1), x2up1(:,1:n2) ];
      x1p2  = Var1SimPs(Ma,epsilon*C',T)      + repmat(xmeana',T,1);
      x2up1 = x1p2*Na';
      Xa    = [ x1p2(:,1:n1), x2up1(:,1:n2) ];
    elseif ModelType == 3;
      x1 = Var1SimPs(Mw,epsilon*C(1:n1,:)',T) + repmat(xmeanw',T,1);
      Xw = [ x1, x1*Nw' ];
      x1 = Var1SimPs(Ma,epsilon*C(1:n1,:)',T) + repmat(xmeana',T,1);
      Xa = [ x1, x1*Nw' ];
    elseif ModelType == 4;
      x1 = Var1SimPs(Mw,epsilon*C(1:n1,:)',T) + repmat(xmeanw',T,1);
      Xw = [x1, x1*Nw'];
      x1 = Var1SimPs(Ma,epsilon*C(1:n1,:)',T) + repmat(xmeana',T,1);
      Xa = [x1, x1*Nw'];
    end;
    vw = -Xw*Fv';
    va = -Xa*Fv';
    ra = sum(sum(va.*va,2))/2 - sum(sum(va.*epsilon(2:T+1,:),2));  %approximating model
    rw = sum(sum(vw.*vw,2))/2 + sum(sum(vw.*epsilon(2:T+1,:),2));  %worst case
    rarwN(i,:) = [ (ra<0), (rw<0) ];
  end;

  paw    = mean(rarwN)';                            %frequencies < 0
  rho(k) = mean(paw);                               %(pa + pw)/2
  JcM(k,:) = [J0, J0a];                            %loss fn [worst, approx]

  disp(' ');disp('sigma, theta, and rho ');
  disp( sprintf('%8.4f ',[sigma(k), -1/sigma(k), rho(k)]) );

  if k == 1
    assert(rho(k) < 0.20, "DEP > 0.20! Please start with a smaller value of sigma.")
  end

  theta = 0;
  if k > 1
    if rho(k) > 0.20
      diffs = [(rho(k-1)-0.2)^2, (rho(k)-0.2)^2];
      [minval, minIdx] = min(diffs);
      kval = k - 2 + minIdx; % kval = k-1 if minIdx==1, kval=k if minIdx==2.
      rhoval = rho(kval);
      JcMval = JcM(kval, :);
      theta  = - 1/sigma(kval);
      break
    end
  end
end;                     %end loop over theta = -1/sigma

fprintf('theta = %.0f \n', theta)
assert(theta ~= 0, "Not found theta. Please put wider range of sigma.")

end
%-----------------------------------------------------------------------